/*
 * Copyright (C) 2015-2021 Swift Navigation Inc.
 * Contact: https://support.swiftnav.com
 *
 * This source is subject to the license found in the file 'LICENSE' which must
 * be be distributed together with this source. All other rights reserved.
 *
 * THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
 * EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
 */

/*****************************************************************************
 * Automatically generated from yaml/swiftnav/sbp/navigation.yaml
 * with generate.py. Please do not hand edit!
 *****************************************************************************/

/** \defgroup navigation Navigation
 *
 * Geodetic navigation messages reporting GPS time, position, velocity, and
 * baseline position solutions. For position solutions, these messages define
 * several different position solutions: single-point (SPP), RTK, and pseudo-
 * absolute position solutions.
 *
 * The SPP is the standalone, absolute GPS position solution using only a
 * single receiver. The RTK solution is the differential GPS solution, which
 * can use either a fixed/integer or floating carrier phase ambiguity. The
 * pseudo-absolute position solution uses a user-provided, well-surveyed base
 * station position (if available) and the RTK solution in tandem.
 *
 * When the inertial navigation mode indicates that the IMU is used, all
 * messages are reported in the vehicle body frame as defined by device
 * settings.  By default, the vehicle body frame is configured to be
 * coincident with the antenna phase center.  When there is no inertial
 * navigation, the solution will be reported at the phase center of the
 * antenna. There is no inertial navigation capability on Piksi Multi or Duro.
 *
 * The tow field, when valid, is most often the Time of Measurement. When this
 * is the case, the 5th bit of flags is set to the default value of 0. When
 * this is not the case, the tow may be a time of arrival or a local system
 * timestamp, irrespective of the time reference (GPS Week or else), but not a
 * Time of Measurement.
 * \{ */

#ifndef LIBSBP_LEGACY_NAVIGATION_MESSAGES_H
#define LIBSBP_LEGACY_NAVIGATION_MESSAGES_H

#include <libsbp/common.h>

#include <libsbp/navigation_macros.h>

SBP_PACK_START

/** GPS Time (GNSS + inertial)
 *
 * This message reports the GPS time, representing the time since the GPS
 * epoch began on midnight January 6, 1980 UTC. GPS time counts the weeks and
 * seconds of the week. The weeks begin at the Saturday/Sunday transition. GPS
 * week 0 began at the beginning of the GPS time scale.
 *
 * Within each week number, the GPS time of the week is between between 0 and
 * 604800 seconds (=60*60*24*7). Note that GPS time does not accumulate leap
 * seconds, and as of now, has a small offset from UTC. In a message stream,
 * this message precedes a set of other navigation messages referenced to the
 * same time (but lacking the ns field) and indicates a more precise time of
 * these messages.
 *
 * The values in this message are from GNSS measurements fused with inertial
 * measurements. To get values from GNSS measurements only use
 * MSG_GPS_TIME_GNSS.
 */

typedef struct SBP_ATTR_PACKED {
  u16 wn;  /**< GPS week number [weeks] */
  u32 tow; /**< GPS time of week rounded to the nearest millisecond [ms] */
  s32 ns_residual; /**< Nanosecond residual of millisecond-rounded TOW
                        (ranges from -500000 to 500000) [ns] */
  u8 flags;        /**< Status flags (reserved) */
} msg_gps_time_t;

/** GNSS-only GPS Time
 *
 * This message reports the GPS time, representing the time since the GPS
 * epoch began on midnight January 6, 1980 UTC. GPS time counts the weeks and
 * seconds of the week. The weeks begin at the Saturday/Sunday transition. GPS
 * week 0 began at the beginning of the GPS time scale.
 *
 * Within each week number, the GPS time of the week is between between 0 and
 * 604800 seconds (=60*60*24*7). Note that GPS time does not accumulate leap
 * seconds, and as of now, has a small offset from UTC. In a message stream,
 * this message precedes a set of other navigation messages referenced to the
 * same time (but lacking the ns field) and indicates a more precise time of
 * these messages.
 *
 * The values in this message are from GNSS measurements only. To get values
 * fused with inertial measurements use MSG_GPS_TIME.
 */

typedef struct SBP_ATTR_PACKED {
  u16 wn;  /**< GPS week number [weeks] */
  u32 tow; /**< GPS time of week rounded to the nearest millisecond [ms] */
  s32 ns_residual; /**< Nanosecond residual of millisecond-rounded TOW
                        (ranges from -500000 to 500000) [ns] */
  u8 flags;        /**< Status flags (reserved) */
} msg_gps_time_gnss_t;

/** UTC Time
 *
 * This message reports the Universal Coordinated Time (UTC).  Note the flags
 * which indicate the source of the UTC offset value and source of the time
 * fix.
 *
 * The values in this message are from GNSS measurements fused with inertial
 * measurements. To get values from GNSS measurements only use
 * MSG_UTC_TIME_GNSS.
 */

typedef struct SBP_ATTR_PACKED {
  u8 flags;   /**< Indicates source and time validity */
  u32 tow;    /**< GPS time of week rounded to the nearest millisecond [ms] */
  u16 year;   /**< Year [year] */
  u8 month;   /**< Month (range 1 .. 12) [months] */
  u8 day;     /**< days in the month (range 1-31) [day] */
  u8 hours;   /**< hours of day (range 0-23) [hours] */
  u8 minutes; /**< minutes of hour (range 0-59) [minutes] */
  u8 seconds; /**< seconds of minute (range 0-60) rounded down [seconds] */
  u32 ns;     /**< nanoseconds of second (range 0-999999999) [nanoseconds] */
} msg_utc_time_t;

/** GNSS-only UTC Time
 *
 * This message reports the Universal Coordinated Time (UTC).  Note the flags
 * which indicate the source of the UTC offset value and source of the time
 * fix.
 *
 * The values in this message are from GNSS measurements only. To get values
 * fused with inertial measurements use MSG_UTC_TIME.
 */

typedef struct SBP_ATTR_PACKED {
  u8 flags;   /**< Indicates source and time validity */
  u32 tow;    /**< GPS time of week rounded to the nearest millisecond [ms] */
  u16 year;   /**< Year [year] */
  u8 month;   /**< Month (range 1 .. 12) [months] */
  u8 day;     /**< days in the month (range 1-31) [day] */
  u8 hours;   /**< hours of day (range 0-23) [hours] */
  u8 minutes; /**< minutes of hour (range 0-59) [minutes] */
  u8 seconds; /**< seconds of minute (range 0-60) rounded down [seconds] */
  u32 ns;     /**< nanoseconds of second (range 0-999999999) [nanoseconds] */
} msg_utc_time_gnss_t;

/** GNSS-only Dilution of Precision
 *
 * This dilution of precision (DOP) message describes the effect of navigation
 * satellite geometry on positional measurement precision.  The flags field
 * indicated whether the DOP reported corresponds to differential or SPP
 * solution.
 *
 * The values in this message are from GNSS measurements only.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;  /**< GPS Time of Week [ms] */
  u16 gdop; /**< Geometric Dilution of Precision [0.01] */
  u16 pdop; /**< Position Dilution of Precision [0.01] */
  u16 tdop; /**< Time Dilution of Precision [0.01] */
  u16 hdop; /**< Horizontal Dilution of Precision [0.01] */
  u16 vdop; /**< Vertical Dilution of Precision [0.01] */
  u8 flags; /**< Indicates the position solution with which the DOPS
                 message corresponds */
} msg_dops_t;

/** Position in ECEF
 *
 * The position solution message reports absolute Earth Centered Earth Fixed
 * (ECEF) coordinates and the status (single point vs pseudo-absolute RTK) of
 * the position solution. If the rover receiver knows the surveyed position of
 * the base station and has an RTK solution, this reports a pseudo-absolute
 * position solution using the base station position and the rover's RTK
 * baseline vector. The full GPS time is given by the preceding MSG_GPS_TIME
 * with the matching time-of-week (tow).
 *
 * The values in this message are from GNSS measurements fused with inertial
 * measurements. To get values from GNSS measurements only use
 * MSG_POS_ECEF_GNSS.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;      /**< GPS Time of Week [ms] */
  double x;     /**< ECEF X coordinate [m] */
  double y;     /**< ECEF Y coordinate [m] */
  double z;     /**< ECEF Z coordinate [m] */
  u16 accuracy; /**< Position estimated standard deviation [mm] */
  u8 n_sats;    /**< Number of satellites used in solution */
  u8 flags;     /**< Status flags */
} msg_pos_ecef_t;

/** Position in ECEF with Covariances
 *
 * The position solution message reports absolute Earth Centered Earth Fixed
 * (ECEF) coordinates and the status (single point vs pseudo-absolute RTK) of
 * the position solution. The message also reports the upper triangular
 * portion of the 3x3 covariance matrix. If the receiver knows the surveyed
 * position of the base station and has an RTK solution, this reports a
 * pseudo-absolute position solution using the base station position and the
 * rover's RTK baseline vector. The full GPS time is given by the preceding
 * MSG_GPS_TIME with the matching time-of-week (tow).
 *
 * The values in this message are from GNSS measurements fused with inertial
 * measurements. To get values from GNSS measurements only use
 * MSG_POS_ECEF_COV_GNSS.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;       /**< GPS Time of Week [ms] */
  double x;      /**< ECEF X coordinate [m] */
  double y;      /**< ECEF Y coordinate [m] */
  double z;      /**< ECEF Z coordinate [m] */
  float cov_x_x; /**< Estimated variance of x [m^2] */
  float cov_x_y; /**< Estimated covariance of x and y [m^2] */
  float cov_x_z; /**< Estimated covariance of x and z [m^2] */
  float cov_y_y; /**< Estimated variance of y [m^2] */
  float cov_y_z; /**< Estimated covariance of y and z [m^2] */
  float cov_z_z; /**< Estimated variance of z [m^2] */
  u8 n_sats;     /**< Number of satellites used in solution */
  u8 flags;      /**< Status flags */
} msg_pos_ecef_cov_t;

/** Geodetic Position
 *
 * This position solution message reports the absolute geodetic coordinates
 * and the status (single point vs pseudo-absolute RTK) of the position
 * solution. If the rover receiver knows the surveyed position of the base
 * station and has an RTK solution, this reports a pseudo-absolute position
 * solution using the base station position and the rover's RTK baseline
 * vector. The full GPS time is given by the preceding MSG_GPS_TIME with the
 * matching time-of-week (tow).
 *
 * The values in this message are from GNSS measurements fused with inertial
 * measurements. To get values from GNSS measurements only use
 * MSG_POS_LLH_GNSS.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;        /**< GPS Time of Week [ms] */
  double lat;     /**< Latitude [deg] */
  double lon;     /**< Longitude [deg] */
  double height;  /**< Height above WGS84 ellipsoid [m] */
  u16 h_accuracy; /**< Horizontal position estimated standard deviation [mm] */
  u16 v_accuracy; /**< Vertical position estimated standard deviation [mm] */
  u8 n_sats;      /**< Number of satellites used in solution. */
  u8 flags;       /**< Status flags */
} msg_pos_llh_t;

/** Geodetic Position with Covariances
 *
 * This position solution message reports the absolute geodetic coordinates
 * and the status (single point vs pseudo-absolute RTK) of the position
 * solution as well as the upper triangle of the 3x3 covariance matrix.  The
 * position information and Fix Mode flags follow the MSG_POS_LLH message.
 * Since the covariance matrix is computed in the local-level North, East,
 * Down frame, the covariance terms follow that convention. Thus, covariances
 * are reported against the "downward" measurement and care should be taken
 * with the sign convention.
 *
 * The values in this message are from GNSS measurements fused with inertial
 * measurements. To get values from GNSS measurements only use
 * MSG_POS_LLH_COV_GNSS.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;       /**< GPS Time of Week [ms] */
  double lat;    /**< Latitude [deg] */
  double lon;    /**< Longitude [deg] */
  double height; /**< Height above WGS84 ellipsoid [m] */
  float cov_n_n; /**< Estimated variance of northing [m^2] */
  float cov_n_e; /**< Covariance of northing and easting [m^2] */
  float cov_n_d; /**< Covariance of northing and downward measurement [m^2] */
  float cov_e_e; /**< Estimated variance of easting [m^2] */
  float cov_e_d; /**< Covariance of easting and downward measurement [m^2] */
  float cov_d_d; /**< Estimated variance of downward measurement [m^2] */
  u8 n_sats;     /**< Number of satellites used in solution. */
  u8 flags;      /**< Status flags */
} msg_pos_llh_cov_t;

typedef struct SBP_ATTR_PACKED {
  float semi_major;  /**< The semi major axis of the estimated horizontal
                          error ellipse at the user-configured confidence
                          level; zero implies invalid. [m] */
  float semi_minor;  /**< The semi minor axis of the estimated horizontal
                          error ellipse at the user-configured confidence
                          level; zero implies invalid. [m] */
  float orientation; /**< The orientation of the semi major axis of the
                          estimated horizontal error ellipse with respect
                          to North. [deg] */
} estimated_horizontal_error_ellipse_t;

/** Geodetic Position and Accuracy
 *
 * This position solution message reports the absolute geodetic coordinates
 * and the status (single point vs pseudo-absolute RTK) of the position
 * solution as well as the estimated horizontal, vertical, cross-track and
 * along-track errors.  The position information and Fix Mode flags  follow
 * the MSG_POS_LLH message. Since the covariance matrix is computed in the
 * local-level North, East, Down frame, the estimated error terms follow that
 * convention.
 *
 * The estimated errors are reported at a user-configurable confidence level.
 * The user-configured percentile is encoded in the percentile field.
 *
 * The values in this message are from GNSS measurements fused with inertial
 * measurements.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;                   /**< GPS Time of Week [ms] */
  double lat;                /**< Latitude [deg] */
  double lon;                /**< Longitude [deg] */
  double height;             /**< Height above WGS84 ellipsoid [m] */
  double orthometric_height; /**< Height above the geoid (i.e. height
                                  above mean sea level). See
                                  confidence_and_geoid for geoid model
                                  used. [m] */
  float h_accuracy;          /**< Estimated horizontal error at the user-
                                  configured confidence level; zero
                                  implies invalid. [m] */
  float v_accuracy;          /**< Estimated vertical error at the user-
                                  configured confidence level; zero
                                  implies invalid. [m] */
  float ct_accuracy;         /**< Estimated cross-track error at the user-
                                  configured confidence level; zero
                                  implies invalid. [m] */
  float at_accuracy;         /**< Estimated along-track error at the user-
                                  configured confidence level; zero
                                  implies invalid. [m] */
  estimated_horizontal_error_ellipse_t h_ellipse; /**< The
                                                       estimated
                                                       horizontal
                                                       error
                                                       ellipse
                                                       at the
                                                       user-
                                                       configured
                                                       confidence
                                                       level. */
  u8 confidence_and_geoid; /**< The lower bits describe the configured
                                confidence level for the estimated position
                                error. The middle bits describe the geoid
                                model used to calculate the orthometric
                                height. */
  u8 n_sats;               /**< Number of satellites used in solution. */
  u8 flags;                /**< Status flags */
} msg_pos_llh_acc_t;

/** GNSS-only Baseline Position in ECEF
 *
 * This message reports the baseline solution in Earth Centered Earth Fixed
 * (ECEF) coordinates. This baseline is the relative vector distance from the
 * base station to the rover receiver. The full GPS time is given by the
 * preceding MSG_GPS_TIME with the matching time-of-week (tow).
 *
 * The values in this message are from GNSS measurements only.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;      /**< GPS Time of Week [ms] */
  s32 x;        /**< Baseline ECEF X coordinate [mm] */
  s32 y;        /**< Baseline ECEF Y coordinate [mm] */
  s32 z;        /**< Baseline ECEF Z coordinate [mm] */
  u16 accuracy; /**< Position estimated standard deviation [mm] */
  u8 n_sats;    /**< Number of satellites used in solution */
  u8 flags;     /**< Status flags */
} msg_baseline_ecef_t;

/** GNSS-only Baseline in NED
 *
 * This message reports the baseline solution in North East Down (NED)
 * coordinates. This baseline is the relative vector distance from the base
 * station to the rover receiver, and NED coordinate system is defined at the
 * local WGS84 tangent plane centered at the base station position.  The full
 * GPS time is given by the preceding MSG_GPS_TIME with the matching time-of-
 * week (tow).
 *
 * The values in this message are from GNSS measurements only.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;        /**< GPS Time of Week [ms] */
  s32 n;          /**< Baseline North coordinate [mm] */
  s32 e;          /**< Baseline East coordinate [mm] */
  s32 d;          /**< Baseline Down coordinate [mm] */
  u16 h_accuracy; /**< Horizontal position estimated standard deviation [mm] */
  u16 v_accuracy; /**< Vertical position estimated standard deviation [mm] */
  u8 n_sats;      /**< Number of satellites used in solution */
  u8 flags;       /**< Status flags */
} msg_baseline_ned_t;

/** Velocity in ECEF
 *
 * This message reports the velocity in Earth Centered Earth Fixed (ECEF)
 * coordinates. The full GPS time is given by the preceding MSG_GPS_TIME with
 * the matching time-of-week (tow).
 *
 * The values in this message are from GNSS measurements fused with inertial
 * measurements. To get values from GNSS measurements only use
 * MSG_VEL_ECEF_GNSS.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;      /**< GPS Time of Week [ms] */
  s32 x;        /**< Velocity ECEF X coordinate [mm/s] */
  s32 y;        /**< Velocity ECEF Y coordinate [mm/s] */
  s32 z;        /**< Velocity ECEF Z coordinate [mm/s] */
  u16 accuracy; /**< Velocity estimated standard deviation [mm/s] */
  u8 n_sats;    /**< Number of satellites used in solution */
  u8 flags;     /**< Status flags */
} msg_vel_ecef_t;

/** Velocity in ECEF with Covariances
 *
 * This message reports the velocity in Earth Centered Earth Fixed (ECEF)
 * coordinates. The full GPS time is given by the preceding MSG_GPS_TIME with
 * the matching time-of-week (tow).
 *
 * The values in this message are from GNSS measurements fused with inertial
 * measurements. To get values from GNSS measurements only use
 * MSG_VEL_ECEF_COV_GNSS.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;       /**< GPS Time of Week [ms] */
  s32 x;         /**< Velocity ECEF X coordinate [mm/s] */
  s32 y;         /**< Velocity ECEF Y coordinate [mm/s] */
  s32 z;         /**< Velocity ECEF Z coordinate [mm/s] */
  float cov_x_x; /**< Estimated variance of x [m^2/s^2] */
  float cov_x_y; /**< Estimated covariance of x and y [m^2/s^2] */
  float cov_x_z; /**< Estimated covariance of x and z [m^2/s^2] */
  float cov_y_y; /**< Estimated variance of y [m^2/s^2] */
  float cov_y_z; /**< Estimated covariance of y and z [m^2/s^2] */
  float cov_z_z; /**< Estimated variance of z [m^2/s^2] */
  u8 n_sats;     /**< Number of satellites used in solution */
  u8 flags;      /**< Status flags */
} msg_vel_ecef_cov_t;

/** Velocity in NED
 *
 * This message reports the velocity in local North East Down (NED)
 * coordinates. The NED coordinate system is defined as the local WGS84
 * tangent plane centered at the current position. The full GPS time is given
 * by the preceding MSG_GPS_TIME with the matching time-of-week (tow).
 *
 * The values in this message are from GNSS measurements fused with inertial
 * measurements. To get values from GNSS measurements only use
 * MSG_VEL_NED_GNSS.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;        /**< GPS Time of Week [ms] */
  s32 n;          /**< Velocity North coordinate [mm/s] */
  s32 e;          /**< Velocity East coordinate [mm/s] */
  s32 d;          /**< Velocity Down coordinate [mm/s] */
  u16 h_accuracy; /**< Horizontal velocity estimated standard deviation [mm/s]
                   */
  u16 v_accuracy; /**< Vertical velocity estimated standard deviation [mm/s] */
  u8 n_sats;      /**< Number of satellites used in solution */
  u8 flags;       /**< Status flags */
} msg_vel_ned_t;

/** Velocity in NED with Covariances
 *
 * This message reports the velocity in local North East Down (NED)
 * coordinates. The NED coordinate system is defined as the local WGS84
 * tangent plane centered at the current position. The full GPS time is given
 * by the preceding MSG_GPS_TIME with the matching time-of-week (tow). This
 * message is similar to the MSG_VEL_NED, but it includes the upper triangular
 * portion of the 3x3 covariance matrix.
 *
 * The values in this message are from GNSS measurements fused with inertial
 * measurements. To get values from GNSS measurements only use
 * MSG_VEL_NED_COV_GNSS.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;       /**< GPS Time of Week [ms] */
  s32 n;         /**< Velocity North coordinate [mm/s] */
  s32 e;         /**< Velocity East coordinate [mm/s] */
  s32 d;         /**< Velocity Down coordinate [mm/s] */
  float cov_n_n; /**< Estimated variance of northward measurement [m^2] */
  float cov_n_e; /**< Covariance of northward and eastward measurement [m^2] */
  float cov_n_d; /**< Covariance of northward and downward measurement [m^2] */
  float cov_e_e; /**< Estimated variance of eastward measurement [m^2] */
  float cov_e_d; /**< Covariance of eastward and downward measurement [m^2] */
  float cov_d_d; /**< Estimated variance of downward measurement [m^2] */
  u8 n_sats;     /**< Number of satellites used in solution */
  u8 flags;      /**< Status flags */
} msg_vel_ned_cov_t;

/** GNSS-only Position in ECEF
 *
 * The position solution message reports absolute Earth Centered Earth Fixed
 * (ECEF) coordinates and the status (single point vs pseudo-absolute RTK) of
 * the position solution. If the rover receiver knows the surveyed position of
 * the base station and has an RTK solution, this reports a pseudo-absolute
 * position solution using the base station position and the rover's RTK
 * baseline vector. The full GPS time is given by the preceding
 * MSG_GPS_TIME_GNSS with the matching time-of-week (tow).
 *
 * The values in this message are from GNSS measurements only. To get values
 * fused with inertial measurements use MSG_POS_ECEF.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;      /**< GPS Time of Week [ms] */
  double x;     /**< ECEF X coordinate [m] */
  double y;     /**< ECEF Y coordinate [m] */
  double z;     /**< ECEF Z coordinate [m] */
  u16 accuracy; /**< Position estimated standard deviation [mm] */
  u8 n_sats;    /**< Number of satellites used in solution */
  u8 flags;     /**< Status flags */
} msg_pos_ecef_gnss_t;

/** GNSS-only Position in ECEF with Covariances
 *
 * The position solution message reports absolute Earth Centered Earth Fixed
 * (ECEF) coordinates and the status (single point vs pseudo-absolute RTK) of
 * the position solution. The message also reports the upper triangular
 * portion of the 3x3 covariance matrix. If the receiver knows the surveyed
 * position of the base station and has an RTK solution, this reports a
 * pseudo-absolute position solution using the base station position and the
 * rover's RTK baseline vector. The full GPS time is given by the preceding
 * MSG_GPS_TIME_GNSS with the matching time-of-week (tow).
 *
 * The values in this message are from GNSS measurements only. To get values
 * fused with inertial measurements use MSG_POS_ECEF_COV.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;       /**< GPS Time of Week [ms] */
  double x;      /**< ECEF X coordinate [m] */
  double y;      /**< ECEF Y coordinate [m] */
  double z;      /**< ECEF Z coordinate [m] */
  float cov_x_x; /**< Estimated variance of x [m^2] */
  float cov_x_y; /**< Estimated covariance of x and y [m^2] */
  float cov_x_z; /**< Estimated covariance of x and z [m^2] */
  float cov_y_y; /**< Estimated variance of y [m^2] */
  float cov_y_z; /**< Estimated covariance of y and z [m^2] */
  float cov_z_z; /**< Estimated variance of z [m^2] */
  u8 n_sats;     /**< Number of satellites used in solution */
  u8 flags;      /**< Status flags */
} msg_pos_ecef_cov_gnss_t;

/** GNSS-only Geodetic Position
 *
 * This position solution message reports the absolute geodetic coordinates
 * and the status (single point vs pseudo-absolute RTK) of the position
 * solution. If the rover receiver knows the surveyed position of the base
 * station and has an RTK solution, this reports a pseudo-absolute position
 * solution using the base station position and the rover's RTK baseline
 * vector. The full GPS time is given by the preceding MSG_GPS_TIME_GNSS with
 * the matching time-of-week (tow).
 *
 * The values in this message are from GNSS measurements only. To get values
 * fused with inertial measurements use MSG_POS_LLH.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;        /**< GPS Time of Week [ms] */
  double lat;     /**< Latitude [deg] */
  double lon;     /**< Longitude [deg] */
  double height;  /**< Height above WGS84 ellipsoid [m] */
  u16 h_accuracy; /**< Horizontal position estimated standard deviation [mm] */
  u16 v_accuracy; /**< Vertical position estimated standard deviation [mm] */
  u8 n_sats;      /**< Number of satellites used in solution. */
  u8 flags;       /**< Status flags */
} msg_pos_llh_gnss_t;

/** GNSS-only Geodetic Position with Covariances
 *
 * This position solution message reports the absolute geodetic coordinates
 * and the status (single point vs pseudo-absolute RTK) of the position
 * solution as well as the upper triangle of the 3x3 covariance matrix.  The
 * position information and Fix Mode flags should follow the MSG_POS_LLH_GNSS
 * message.  Since the covariance matrix is computed in the local-level North,
 * East, Down frame, the covariance terms follow with that convention. Thus,
 * covariances are reported against the "downward" measurement and care should
 * be taken with the sign convention.
 *
 * The values in this message are from GNSS measurements only. To get values
 * fused with inertial measurements use MSG_POS_LLH_COV.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;       /**< GPS Time of Week [ms] */
  double lat;    /**< Latitude [deg] */
  double lon;    /**< Longitude [deg] */
  double height; /**< Height above WGS84 ellipsoid [m] */
  float cov_n_n; /**< Estimated variance of northing [m^2] */
  float cov_n_e; /**< Covariance of northing and easting [m^2] */
  float cov_n_d; /**< Covariance of northing and downward measurement [m^2] */
  float cov_e_e; /**< Estimated variance of easting [m^2] */
  float cov_e_d; /**< Covariance of easting and downward measurement [m^2] */
  float cov_d_d; /**< Estimated variance of downward measurement [m^2] */
  u8 n_sats;     /**< Number of satellites used in solution. */
  u8 flags;      /**< Status flags */
} msg_pos_llh_cov_gnss_t;

/** GNSS-only Velocity in ECEF
 *
 * This message reports the velocity in Earth Centered Earth Fixed (ECEF)
 * coordinates. The full GPS time is given by the preceding MSG_GPS_TIME_GNSS
 * with the matching time-of-week (tow).
 *
 * The values in this message are from GNSS measurements only. To get values
 * fused with inertial measurements use MSG_VEL_ECEF.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;      /**< GPS Time of Week [ms] */
  s32 x;        /**< Velocity ECEF X coordinate [mm/s] */
  s32 y;        /**< Velocity ECEF Y coordinate [mm/s] */
  s32 z;        /**< Velocity ECEF Z coordinate [mm/s] */
  u16 accuracy; /**< Velocity estimated standard deviation [mm/s] */
  u8 n_sats;    /**< Number of satellites used in solution */
  u8 flags;     /**< Status flags */
} msg_vel_ecef_gnss_t;

/** GNSS-only Velocity in ECEF with Covariances
 *
 * This message reports the velocity in Earth Centered Earth Fixed (ECEF)
 * coordinates. The full GPS time is given by the preceding MSG_GPS_TIME_GNSS
 * with the matching time-of-week (tow).
 *
 * The values in this message are from GNSS measurements only. To get values
 * fused with inertial measurements use MSG_VEL_ECEF_COV.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;       /**< GPS Time of Week [ms] */
  s32 x;         /**< Velocity ECEF X coordinate [mm/s] */
  s32 y;         /**< Velocity ECEF Y coordinate [mm/s] */
  s32 z;         /**< Velocity ECEF Z coordinate [mm/s] */
  float cov_x_x; /**< Estimated variance of x [m^2/s^2] */
  float cov_x_y; /**< Estimated covariance of x and y [m^2/s^2] */
  float cov_x_z; /**< Estimated covariance of x and z [m^2/s^2] */
  float cov_y_y; /**< Estimated variance of y [m^2/s^2] */
  float cov_y_z; /**< Estimated covariance of y and z [m^2/s^2] */
  float cov_z_z; /**< Estimated variance of z [m^2/s^2] */
  u8 n_sats;     /**< Number of satellites used in solution */
  u8 flags;      /**< Status flags */
} msg_vel_ecef_cov_gnss_t;

/** GNSS-only Velocity in NED
 *
 * This message reports the velocity in local North East Down (NED)
 * coordinates. The NED coordinate system is defined as the local WGS84
 * tangent plane centered at the current position. The full GPS time is given
 * by the preceding MSG_GPS_TIME_GNSS with the matching time-of-week (tow).
 *
 * The values in this message are from GNSS measurements only. To get values
 * fused with inertial measurements use MSG_VEL_NED.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;        /**< GPS Time of Week [ms] */
  s32 n;          /**< Velocity North coordinate [mm/s] */
  s32 e;          /**< Velocity East coordinate [mm/s] */
  s32 d;          /**< Velocity Down coordinate [mm/s] */
  u16 h_accuracy; /**< Horizontal velocity estimated standard deviation [mm/s]
                   */
  u16 v_accuracy; /**< Vertical velocity estimated standard deviation [mm/s] */
  u8 n_sats;      /**< Number of satellites used in solution */
  u8 flags;       /**< Status flags */
} msg_vel_ned_gnss_t;

/** GNSS-only Velocity in NED with Covariances
 *
 * This message reports the velocity in local North East Down (NED)
 * coordinates. The NED coordinate system is defined as the local WGS84
 * tangent plane centered at the current position. The full GPS time is given
 * by the preceding MSG_GPS_TIME_GNSS with the matching time-of-week (tow).
 * This message is similar to the MSG_VEL_NED_GNSS, but it includes the upper
 * triangular portion of the 3x3 covariance matrix.
 *
 * The values in this message are from GNSS measurements only. To get values
 * fused with inertial measurements use MSG_VEL_NED_COV.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;       /**< GPS Time of Week [ms] */
  s32 n;         /**< Velocity North coordinate [mm/s] */
  s32 e;         /**< Velocity East coordinate [mm/s] */
  s32 d;         /**< Velocity Down coordinate [mm/s] */
  float cov_n_n; /**< Estimated variance of northward measurement [m^2] */
  float cov_n_e; /**< Covariance of northward and eastward measurement [m^2] */
  float cov_n_d; /**< Covariance of northward and downward measurement [m^2] */
  float cov_e_e; /**< Estimated variance of eastward measurement [m^2] */
  float cov_e_d; /**< Covariance of eastward and downward measurement [m^2] */
  float cov_d_d; /**< Estimated variance of downward measurement [m^2] */
  u8 n_sats;     /**< Number of satellites used in solution */
  u8 flags;      /**< Status flags */
} msg_vel_ned_cov_gnss_t;

/** Velocity in User Frame
 *
 * This message reports the velocity in the Vehicle Body Frame. By convention,
 * the x-axis should point out the nose of the vehicle and represent the
 * forward direction, while as the y-axis should point out the right hand side
 * of the vehicle. Since this is a right handed system, z should point out the
 * bottom of the vehicle. The orientation and origin of the Vehicle Body Frame
 * are specified via the device settings. The full GPS time is given by the
 * preceding MSG_GPS_TIME with the matching time-of-week (tow). This message
 * is only produced by inertial versions of Swift products and is not
 * available from Piksi Multi or Duro.
 *
 * The values in this message are from GNSS measurements fused with inertial
 * measurements.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;       /**< GPS Time of Week [ms] */
  s32 x;         /**< Velocity in x direction [mm/s] */
  s32 y;         /**< Velocity in y direction [mm/s] */
  s32 z;         /**< Velocity in z direction [mm/s] */
  float cov_x_x; /**< Estimated variance of x [m^2] */
  float cov_x_y; /**< Covariance of x and y [m^2] */
  float cov_x_z; /**< Covariance of x and z [m^2] */
  float cov_y_y; /**< Estimated variance of y [m^2] */
  float cov_y_z; /**< Covariance of y and z [m^2] */
  float cov_z_z; /**< Estimated variance of z [m^2] */
  u8 n_sats;     /**< Number of satellites used in solution */
  u8 flags;      /**< Status flags */
} msg_vel_body_t;

/** Velocity expressed as course over ground
 *
 * This message reports the receiver course over ground (COG) and speed over
 * ground (SOG) based on the horizontal (N-E) components of the NED velocity
 * vector. It also includes the vertical velocity coordinate. A flag is
 * provided to indicate whether the COG value has been frozen. When the flag
 * is set to true, the COG field is set to its last valid value until the
 * system exceeds a minimum velocity threshold. No other fields are affected
 * by this flag. The NED coordinate system is defined as the local WGS84
 * tangent plane centered at the current position. The full GPS time is given
 * by the preceding MSG_GPS_TIME with the matching time-of-week (tow). Note:
 * course over ground represents the receiver's direction of travel, but not
 * necessarily the device heading.
 *
 * The values in this message are from GNSS measurements fused with inertial
 * measurements.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow; /**< GPS Time of Week [ms] */
  u32 cog; /**< Course over ground relative to north direction [microdegrees] */
  u32 sog; /**< Speed over ground (based on horizontal velocity) [mm/s] */
  s32 v_up;          /**< Vertical velocity component (positive up) [mm/s] */
  u32 cog_accuracy;  /**< Course over ground estimated standard deviation
                        [microdegrees] */
  u32 sog_accuracy;  /**< Speed over ground estimated standard deviation [mm/s]
                      */
  u32 v_up_accuracy; /**< Vertical velocity estimated standard deviation [mm/s]
                      */
  u16 flags;         /**< Status flags */
} msg_vel_cog_t;

/** Age of corrections
 *
 * This message reports the Age of the corrections used for the current
 * Differential solution.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow; /**< GPS Time of Week [ms] */
  u16 age; /**< Age of the corrections (0xFFFF indicates invalid) [deciseconds]
            */
} msg_age_corrections_t;

/** Deprecated
 *
 * Deprecated.
 */

typedef struct SBP_ATTR_PACKED {
  u16 wn;  /**< GPS week number [weeks] */
  u32 tow; /**< GPS time of week rounded to the nearest millisecond [ms] */
  s32 ns_residual; /**< Nanosecond residual of millisecond-rounded TOW
                        (ranges from -500000 to 500000) [ns] */
  u8 flags;        /**< Status flags (reserved) */
} msg_gps_time_dep_a_t;

/** Deprecated
 *
 * Deprecated.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;  /**< GPS Time of Week [ms] */
  u16 gdop; /**< Geometric Dilution of Precision [0.01] */
  u16 pdop; /**< Position Dilution of Precision [0.01] */
  u16 tdop; /**< Time Dilution of Precision [0.01] */
  u16 hdop; /**< Horizontal Dilution of Precision [0.01] */
  u16 vdop; /**< Vertical Dilution of Precision [0.01] */
} msg_dops_dep_a_t;

/** Deprecated
 *
 * Deprecated.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;      /**< GPS Time of Week [ms] */
  double x;     /**< ECEF X coordinate [m] */
  double y;     /**< ECEF Y coordinate [m] */
  double z;     /**< ECEF Z coordinate [m] */
  u16 accuracy; /**< Position accuracy estimate (not implemented). Defaults
                     to 0. [mm] */
  u8 n_sats;    /**< Number of satellites used in solution */
  u8 flags;     /**< Status flags */
} msg_pos_ecef_dep_a_t;

/** Deprecated
 *
 * Deprecated.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;        /**< GPS Time of Week [ms] */
  double lat;     /**< Latitude [deg] */
  double lon;     /**< Longitude [deg] */
  double height;  /**< Height [m] */
  u16 h_accuracy; /**< Horizontal position accuracy estimate (not
                       implemented). Defaults to 0. [mm] */
  u16 v_accuracy; /**< Vertical position accuracy estimate (not
                       implemented). Defaults to 0. [mm] */
  u8 n_sats;      /**< Number of satellites used in solution. */
  u8 flags;       /**< Status flags */
} msg_pos_llh_dep_a_t;

/** Deprecated
 *
 * Deprecated.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;      /**< GPS Time of Week [ms] */
  s32 x;        /**< Baseline ECEF X coordinate [mm] */
  s32 y;        /**< Baseline ECEF Y coordinate [mm] */
  s32 z;        /**< Baseline ECEF Z coordinate [mm] */
  u16 accuracy; /**< Position accuracy estimate [mm] */
  u8 n_sats;    /**< Number of satellites used in solution */
  u8 flags;     /**< Status flags */
} msg_baseline_ecef_dep_a_t;

/** Deprecated
 *
 * Deprecated.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;        /**< GPS Time of Week [ms] */
  s32 n;          /**< Baseline North coordinate [mm] */
  s32 e;          /**< Baseline East coordinate [mm] */
  s32 d;          /**< Baseline Down coordinate [mm] */
  u16 h_accuracy; /**< Horizontal position accuracy estimate (not
                       implemented). Defaults to 0. [mm] */
  u16 v_accuracy; /**< Vertical position accuracy estimate (not
                       implemented). Defaults to 0. [mm] */
  u8 n_sats;      /**< Number of satellites used in solution */
  u8 flags;       /**< Status flags */
} msg_baseline_ned_dep_a_t;

/** Deprecated
 *
 * Deprecated.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;      /**< GPS Time of Week [ms] */
  s32 x;        /**< Velocity ECEF X coordinate [mm/s] */
  s32 y;        /**< Velocity ECEF Y coordinate [mm/s] */
  s32 z;        /**< Velocity ECEF Z coordinate [mm/s] */
  u16 accuracy; /**< Velocity accuracy estimate (not implemented). Defaults
                     to 0. [mm/s] */
  u8 n_sats;    /**< Number of satellites used in solution */
  u8 flags;     /**< Status flags (reserved) */
} msg_vel_ecef_dep_a_t;

/** Deprecated
 *
 * Deprecated.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;        /**< GPS Time of Week [ms] */
  s32 n;          /**< Velocity North coordinate [mm/s] */
  s32 e;          /**< Velocity East coordinate [mm/s] */
  s32 d;          /**< Velocity Down coordinate [mm/s] */
  u16 h_accuracy; /**< Horizontal velocity accuracy estimate (not
                       implemented). Defaults to 0. [mm/s] */
  u16 v_accuracy; /**< Vertical velocity accuracy estimate (not
                       implemented). Defaults to 0. [mm/s] */
  u8 n_sats;      /**< Number of satellites used in solution */
  u8 flags;       /**< Status flags (reserved) */
} msg_vel_ned_dep_a_t;

/** Deprecated
 *
 * Deprecated.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;     /**< GPS Time of Week [ms] */
  u32 heading; /**< Heading [mdeg] */
  u8 n_sats;   /**< Number of satellites used in solution */
  u8 flags;    /**< Status flags */
} msg_baseline_heading_dep_a_t;

/** Deprecated
 *
 * Deprecated.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;       /**< GPS Time of Week [ms] */
  u16 vpl;       /**< Vertical protection level [cm] */
  u16 hpl;       /**< Horizontal protection level [cm] */
  double lat;    /**< Latitude [deg] */
  double lon;    /**< Longitude [deg] */
  double height; /**< Height [m] */
  u8 flags;      /**< Status flags */
} msg_protection_level_dep_a_t;

/** Computed state and Protection Levels
 *
 * This message reports the protection levels associated to the given state
 * estimate. The full GPS time is given by the preceding MSG_GPS_TIME with the
 * matching time-of-week (tow).
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;       /**< GPS Time of Week [ms] */
  s16 wn;        /**< GPS week number [weeks] */
  u16 hpl;       /**< Horizontal protection level [cm] */
  u16 vpl;       /**< Vertical protection level [cm] */
  u16 atpl;      /**< Along-track position error protection level [cm] */
  u16 ctpl;      /**< Cross-track position error protection level [cm] */
  u16 hvpl;      /**< Protection level for the error vector between estimated
                      and true along/cross track velocity vector [mm/s] */
  u16 vvpl;      /**< Protection level for the velocity in vehicle upright
                      direction (different from vertical direction if on a
                      slope) [mm/s] */
  u16 hopl;      /**< Heading orientation protection level [mdeg] */
  u16 popl;      /**< Pitch orientation protection level [mdeg] */
  u16 ropl;      /**< Roll orientation protection level [mdeg] */
  double lat;    /**< Latitude [deg] */
  double lon;    /**< Longitude [deg] */
  double height; /**< Height [m] */
  s32 v_x;       /**< Velocity in vehicle x direction [mm/s] */
  s32 v_y;       /**< Velocity in vehicle y direction [mm/s] */
  s32 v_z;       /**< Velocity in vehicle z direction [mm/s] */
  s32 roll;      /**< Roll angle [udeg] */
  s32 pitch;     /**< Pitch angle [udeg] */
  s32 heading;   /**< Heading angle [udeg] */
  u32 flags;     /**< Status flags */
} msg_protection_level_t;

/** Leap second SBP message.

 *
 * UTC-GPST leap seconds before and after the most recent (past, or future,
 * for announced insertions) UTC leap second insertion.
 */

typedef struct SBP_ATTR_PACKED {
  s16 reserved_0;  /**< Reserved. */
  s16 reserved_1;  /**< Reserved. */
  s8 reserved_2;   /**< Reserved. */
  s8 count_before; /**< Leap second count before insertion. [s] */
  u16 reserved_3;  /**< Reserved. */
  u16 reserved_4;  /**< Reserved. */
  u16 ref_wn;      /**< Leap second reference GPS week number. [weeks] */
  u8 ref_dn;       /**< Leap second reference day number. [days] */
  s8 count_after;  /**< Leap second count after insertion. [s] */
} msg_utc_leap_second_t;

typedef struct SBP_ATTR_PACKED {
  u8 ssr_iod;   /**< SSR IOD parameter. */
  char sn[32];  /**< Name of source coordinate-system. */
  char tn[32];  /**< Name of target coordinate-system. */
  u8 sin;       /**< System Identification Number. */
  u16 utn;      /**< Utilized Transformation Message. */
  u16 re_t0;    /**< Reference Epoch t0 for transformation parameter
                     set given as Modified Julian Day (MJD) Number
                     minus 44244 days. [1 day] */
  s32 delta_X0; /**< Translation in X for Reference Epoch t0. [0.001 m] */
  s32 delta_Y0; /**< Translation in Y for Reference Epoch t0. [0.001 m] */
  s32 delta_Z0; /**< Translation in Z for Reference Epoch t0. [0.001 m] */
  s32 theta_01; /**< Rotation around the X-axis for Reference Epoch t0. [0.00002
                   "] */
  s32 theta_02; /**< Rotation around the Y-axis for Reference Epoch t0. [0.00002
                   "] */
  s32 theta_03; /**< Rotation around the Z-axis for Reference Epoch t0. [0.00002
                   "] */
  s32 scale;    /**< Scale correction for Reference Epoch t0. [0.00001 ppm] */
  s32 dot_delta_X0; /**< Rate of change of translation in X. [0.00002 m/yr] */
  s32 dot_delta_Y0; /**< Rate of change of translation in Y. [0.00002 m/yr] */
  s32 dot_delta_Z0; /**< Rate of change of translation in Z. [0.00002 m/yr] */
  s32 dot_theta_01; /**< Rate of change of rotation around the X-axis.
                       [0.0000004 "/yr] */
  s32 dot_theta_02; /**< Rate of change of rotation around the Y-axis.
                       [0.0000004 "/yr] */
  s32 dot_theta_03; /**< Rate of change of rotation around the Z-axis.
                       [0.0000004 "/yr] */
  s16 dot_scale; /**< Rate of change of scale correction. [0.0000002 ppm/yr] */
} msg_reference_frame_param_t;

/** Relative Pose
 *
 * This solution message reports the relative pose of a sensor between two
 * time instances. The relative pose comprises of a rotation and a translation
 * which relates the sensor (e.g. camera) frame at a given time (first
 * keyframe) to the sensor frame at another time (second keyframe). The
 * relative translations is a 3x1 vector described in the first keyframe.
 * Relative rotation is described by a quaternion from second keyframe to the
 * first keyframe.
 */

typedef struct SBP_ATTR_PACKED {
  u32 tow;         /**< GPS Time of Week [ms] */
  u8 sensor_id;    /**< ID of the sensor producing this message */
  u32 timestamp_1; /**< Timestamp of first keyframe [ms] */
  u32 timestamp_2; /**< Timestamp of second keyframe [ms] */
  s32 trans[3];    /**< Relative translation [x,y,z] described in first
                        keyframe [mm] */
  s32 w;           /**< Real component of quaternion to describe relative
                        rotation (second to first keyframe) [2^-31] */
  s32 x;           /**< 1st imaginary component of quaternion to describe
                        relative rotation (second to first keyframe) [2^-31] */
  s32 y;           /**< 2nd imaginary component of quaternion to describe
                        relative rotation (second to first keyframe) [2^-31] */
  s32 z;           /**< 3rd imaginary component of quaternion to describe
                        relative rotation (second to first keyframe) [2^-31] */
  float cov_r_x_x; /**< Estimated variance of x (relative translation) [m^2] */
  float cov_r_x_y; /**< Covariance of x and y (relative translation) [m^2] */
  float cov_r_x_z; /**< Covariance of x and z (relative translation) [m^2] */
  float cov_r_y_y; /**< Estimated variance of y (relative translation) [m^2] */
  float cov_r_y_z; /**< Covariance of y and z (relative translation) [m^2] */
  float cov_r_z_z; /**< Estimated variance of z (relative translation) [m^2] */
  float cov_c_x_x; /**< Estimated variance of x (relative rotation) [rad^2] */
  float cov_c_x_y; /**< Covariance of x and y (relative rotation) [rad^2] */
  float cov_c_x_z; /**< Covariance of x and z (relative rotation) [rad^2] */
  float cov_c_y_y; /**< Estimated variance of y (relative rotation) [rad^2] */
  float cov_c_y_z; /**< Covariance of y and z (relative rotation) [rad^2] */
  float cov_c_z_z; /**< Estimated variance of z (relative rotation) [rad^2] */
  u8 flags;        /**< Status flags of relative translation and rotation */
} msg_pose_relative_t;

/** \} */

SBP_PACK_END

#endif /* LIBSBP_LEGACY_NAVIGATION_MESSAGES_H */