/*
 * Copyright (C) 2015-2021 Swift Navigation Inc.
 * Contact: https://support.swiftnav.com
 *
 * This source is subject to the license found in the file 'LICENSE' which must
 * be be distributed together with this source. All other rights reserved.
 *
 * THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
 * EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
 */

// This file was auto-generated from
// spec/tests/yaml/swiftnav/sbp/piksi/test_MsgNetworkStateResp.yaml by
// generate.py. Do not modify by hand!

#include <check.h>
#include <libsbp/legacy/piksi.h>
#include <sbp.h>
#include <stdio.h>   // for debugging
#include <stdlib.h>  // for malloc

static struct {
  u32 n_callbacks_logged;
  u16 sender_id;
  u8 len;
  u8 msg[SBP_MAX_PAYLOAD_LEN];
  void *context;
} last_msg;

static struct {
  u32 n_callbacks_logged;
  u16 sender_id;
  u16 msg_type;
  u8 msg_len;
  u8 msg[SBP_MAX_PAYLOAD_LEN];
  u16 frame_len;
  u8 frame[SBP_MAX_FRAME_LEN];
  void *context;
} last_frame;

static u32 dummy_wr = 0;
static u32 dummy_rd = 0;
static u8 dummy_buff[1024];
static void *last_io_context;

static int DUMMY_MEMORY_FOR_CALLBACKS = 0xdeadbeef;
static int DUMMY_MEMORY_FOR_IO = 0xdead0000;

static void dummy_reset() {
  dummy_rd = dummy_wr = 0;
  memset(dummy_buff, 0, sizeof(dummy_buff));
}

static s32 dummy_write(u8 *buff, u32 n, void *context) {
  last_io_context = context;
  u32 real_n = n;  //(dummy_n > n) ? n : dummy_n;
  memcpy(dummy_buff + dummy_wr, buff, real_n);
  dummy_wr += real_n;
  return real_n;
}

static s32 dummy_read(u8 *buff, u32 n, void *context) {
  last_io_context = context;
  u32 real_n = n;  //(dummy_n > n) ? n : dummy_n;
  memcpy(buff, dummy_buff + dummy_rd, real_n);
  dummy_rd += real_n;
  return real_n;
}

static void logging_reset() {
  memset(&last_msg, 0, sizeof(last_msg));
  memset(&last_frame, 0, sizeof(last_frame));
}

static void msg_callback(u16 sender_id, u8 len, u8 msg[], void *context) {
  last_msg.n_callbacks_logged++;
  last_msg.sender_id = sender_id;
  last_msg.len = len;
  last_msg.context = context;
  memcpy(last_msg.msg, msg, len);
}

static void frame_callback(u16 sender_id, u16 msg_type, u8 msg_len, u8 msg[],
                           u16 frame_len, u8 frame[], void *context) {
  last_frame.n_callbacks_logged++;
  last_frame.sender_id = sender_id;
  last_frame.msg_type = msg_type;
  last_frame.msg_len = msg_len;
  memcpy(last_frame.msg, msg, msg_len);
  last_frame.frame_len = frame_len;
  memcpy(last_frame.frame, frame, frame_len);
  last_frame.context = context;
}

START_TEST(test_legacy_auto_check_sbp_piksi_MsgNetworkStateResp) {
  static sbp_msg_callbacks_node_t n;
  static sbp_msg_callbacks_node_t n2;

  // State of the SBP message parser.
  // Must be statically allocated.
  sbp_state_t sbp_state;

  //
  // Run tests:
  //
  // Test successful parsing of a message
  {
    // SBP parser state must be initialized before sbp_process is called.
    // We re-initialize before every test so that callbacks for the same message
    // types can be
    //  allocated multiple times across different tests.
    sbp_state_init(&sbp_state);

    sbp_state_set_io_context(&sbp_state, &DUMMY_MEMORY_FOR_IO);

    logging_reset();

    sbp_payload_callback_register(&sbp_state, 0xbb, &msg_callback,
                                  &DUMMY_MEMORY_FOR_CALLBACKS, &n);
    sbp_frame_callback_register(&sbp_state, 0xbb, &frame_callback,
                                &DUMMY_MEMORY_FOR_CALLBACKS, &n2);

    u8 encoded_frame[] = {
        85,  187, 0,  40,  15,  50,  143, 241, 84,  180, 152, 194,
        137, 32,  44, 114, 147, 68,  222, 92,  192, 78,  235, 63,
        208, 114, 53, 183, 24,  244, 231, 26,  105, 25,  136, 3,
        105, 102, 48, 0,   0,   0,   0,   0,   0,   0,   0,   0,
        0,   0,   0,  0,   195, 229, 80,  147, 118, 193,
    };

    dummy_reset();

    u8 test_msg_storage[SBP_MAX_PAYLOAD_LEN];
    memset(test_msg_storage, 0, sizeof(test_msg_storage));
    u8 test_msg_len = 0;
    msg_network_state_resp_t *test_msg =
        (msg_network_state_resp_t *)test_msg_storage;
    test_msg_len = sizeof(*test_msg);
    test_msg->flags = 2471552451;
    {
      const char assign_string[] = {(char)105, (char)102, (char)48, (char)0,
                                    (char)0,   (char)0,   (char)0,  (char)0,
                                    (char)0,   (char)0,   (char)0,  (char)0,
                                    (char)0,   (char)0,   (char)0,  (char)0};
      memcpy(test_msg->interface_name, assign_string, sizeof(assign_string));
      if (sizeof(test_msg->interface_name) == 0) {
        test_msg_len += sizeof(assign_string);
      }
    }
    if (sizeof(test_msg->ipv4_address) == 0) {
      // Cope with variable length arrays
      test_msg_len += sizeof(test_msg->ipv4_address[0]);
    }
    test_msg->ipv4_address[0] = 143;
    if (sizeof(test_msg->ipv4_address) == 0) {
      // Cope with variable length arrays
      test_msg_len += sizeof(test_msg->ipv4_address[0]);
    }
    test_msg->ipv4_address[1] = 241;
    if (sizeof(test_msg->ipv4_address) == 0) {
      // Cope with variable length arrays
      test_msg_len += sizeof(test_msg->ipv4_address[0]);
    }
    test_msg->ipv4_address[2] = 84;
    if (sizeof(test_msg->ipv4_address) == 0) {
      // Cope with variable length arrays
      test_msg_len += sizeof(test_msg->ipv4_address[0]);
    }
    test_msg->ipv4_address[3] = 180;
    test_msg->ipv4_mask_size = 152;
    if (sizeof(test_msg->ipv6_address) == 0) {
      // Cope with variable length arrays
      test_msg_len += sizeof(test_msg->ipv6_address[0]);
    }
    test_msg->ipv6_address[0] = 194;
    if (sizeof(test_msg->ipv6_address) == 0) {
      // Cope with variable length arrays
      test_msg_len += sizeof(test_msg->ipv6_address[0]);
    }
    test_msg->ipv6_address[1] = 137;
    if (sizeof(test_msg->ipv6_address) == 0) {
      // Cope with variable length arrays
      test_msg_len += sizeof(test_msg->ipv6_address[0]);
    }
    test_msg->ipv6_address[2] = 32;
    if (sizeof(test_msg->ipv6_address) == 0) {
      // Cope with variable length arrays
      test_msg_len += sizeof(test_msg->ipv6_address[0]);
    }
    test_msg->ipv6_address[3] = 44;
    if (sizeof(test_msg->ipv6_address) == 0) {
      // Cope with variable length arrays
      test_msg_len += sizeof(test_msg->ipv6_address[0]);
    }
    test_msg->ipv6_address[4] = 114;
    if (sizeof(test_msg->ipv6_address) == 0) {
      // Cope with variable length arrays
      test_msg_len += sizeof(test_msg->ipv6_address[0]);
    }
    test_msg->ipv6_address[5] = 147;
    if (sizeof(test_msg->ipv6_address) == 0) {
      // Cope with variable length arrays
      test_msg_len += sizeof(test_msg->ipv6_address[0]);
    }
    test_msg->ipv6_address[6] = 68;
    if (sizeof(test_msg->ipv6_address) == 0) {
      // Cope with variable length arrays
      test_msg_len += sizeof(test_msg->ipv6_address[0]);
    }
    test_msg->ipv6_address[7] = 222;
    if (sizeof(test_msg->ipv6_address) == 0) {
      // Cope with variable length arrays
      test_msg_len += sizeof(test_msg->ipv6_address[0]);
    }
    test_msg->ipv6_address[8] = 92;
    if (sizeof(test_msg->ipv6_address) == 0) {
      // Cope with variable length arrays
      test_msg_len += sizeof(test_msg->ipv6_address[0]);
    }
    test_msg->ipv6_address[9] = 192;
    if (sizeof(test_msg->ipv6_address) == 0) {
      // Cope with variable length arrays
      test_msg_len += sizeof(test_msg->ipv6_address[0]);
    }
    test_msg->ipv6_address[10] = 78;
    if (sizeof(test_msg->ipv6_address) == 0) {
      // Cope with variable length arrays
      test_msg_len += sizeof(test_msg->ipv6_address[0]);
    }
    test_msg->ipv6_address[11] = 235;
    if (sizeof(test_msg->ipv6_address) == 0) {
      // Cope with variable length arrays
      test_msg_len += sizeof(test_msg->ipv6_address[0]);
    }
    test_msg->ipv6_address[12] = 63;
    if (sizeof(test_msg->ipv6_address) == 0) {
      // Cope with variable length arrays
      test_msg_len += sizeof(test_msg->ipv6_address[0]);
    }
    test_msg->ipv6_address[13] = 208;
    if (sizeof(test_msg->ipv6_address) == 0) {
      // Cope with variable length arrays
      test_msg_len += sizeof(test_msg->ipv6_address[0]);
    }
    test_msg->ipv6_address[14] = 114;
    if (sizeof(test_msg->ipv6_address) == 0) {
      // Cope with variable length arrays
      test_msg_len += sizeof(test_msg->ipv6_address[0]);
    }
    test_msg->ipv6_address[15] = 53;
    test_msg->ipv6_mask_size = 183;
    test_msg->rx_bytes = 451408920;
    test_msg->tx_bytes = 59251049;
    sbp_payload_send(&sbp_state, 0xbb, 3880, test_msg_len, test_msg_storage,
                     &dummy_write);

    ck_assert_msg(
        test_msg_len == sizeof(encoded_frame) - 8,
        "Test message has not been generated correctly, or the encoded frame "
        "from the spec is badly defined. Check your test spec");

    ck_assert_msg(dummy_wr == sizeof(encoded_frame),
                  "not enough data was written to dummy_buff");
    ck_assert_msg(memcmp(dummy_buff, encoded_frame, sizeof(encoded_frame)) == 0,
                  "frame was not encoded properly");

    while (dummy_rd < dummy_wr) {
      ck_assert_msg(sbp_process(&sbp_state, &dummy_read) >= SBP_OK,
                    "sbp_process threw an error!");
    }

    ck_assert_msg(last_msg.n_callbacks_logged == 1,
                  "msg_callback: one callback should have been logged");
    ck_assert_msg(last_msg.sender_id == 3880,
                  "msg_callback: sender_id decoded incorrectly");
    ck_assert_msg(last_msg.len == sizeof(encoded_frame) - 8,
                  "msg_callback: len decoded incorrectly");
    ck_assert_msg(
        memcmp(last_msg.msg, encoded_frame + 6, sizeof(encoded_frame) - 8) == 0,
        "msg_callback: test data decoded incorrectly");
    ck_assert_msg(last_msg.context == &DUMMY_MEMORY_FOR_CALLBACKS,
                  "frame_callback: context pointer incorrectly passed");

    ck_assert_msg(last_frame.n_callbacks_logged == 1,
                  "frame_callback: one callback should have been logged");
    ck_assert_msg(last_frame.sender_id == 3880,
                  "frame_callback: sender_id decoded incorrectly");
    ck_assert_msg(last_frame.msg_type == 0xbb,
                  "frame_callback: msg_type decoded incorrectly");
    ck_assert_msg(last_frame.msg_len == sizeof(encoded_frame) - 8,
                  "frame_callback: msg_len decoded incorrectly");
    ck_assert_msg(memcmp(last_frame.msg, encoded_frame + 6,
                         sizeof(encoded_frame) - 8) == 0,
                  "frame_callback: test data decoded incorrectly");
    ck_assert_msg(last_frame.frame_len == sizeof(encoded_frame),
                  "frame_callback: frame_len decoded incorrectly");
    ck_assert_msg(
        memcmp(last_frame.frame, encoded_frame, sizeof(encoded_frame)) == 0,
        "frame_callback: frame decoded incorrectly");
    ck_assert_msg(last_frame.context == &DUMMY_MEMORY_FOR_CALLBACKS,
                  "frame_callback: context pointer incorrectly passed");

    // Cast to expected message type - the +6 byte offset is where the payload
    // starts
    msg_network_state_resp_t *check_msg =
        (msg_network_state_resp_t *)((void *)last_msg.msg);
    // Run tests against fields
    ck_assert_msg(check_msg != 0, "stub to prevent warnings if msg isn't used");
    ck_assert_msg(check_msg->flags == 2471552451,
                  "incorrect value for flags, expected 2471552451, is %d",
                  check_msg->flags);
    {
      const char check_string[] = {(char)105, (char)102, (char)48, (char)0,
                                   (char)0,   (char)0,   (char)0,  (char)0,
                                   (char)0,   (char)0,   (char)0,  (char)0,
                                   (char)0,   (char)0,   (char)0,  (char)0};
      ck_assert_msg(memcmp(check_msg->interface_name, check_string,
                           sizeof(check_string)) == 0,
                    "incorrect value for check_msg->interface_name, expected "
                    "string '%s', is '%s'",
                    check_string, check_msg->interface_name);
    }
    ck_assert_msg(check_msg->ipv4_address[0] == 143,
                  "incorrect value for ipv4_address[0], expected 143, is %d",
                  check_msg->ipv4_address[0]);
    ck_assert_msg(check_msg->ipv4_address[1] == 241,
                  "incorrect value for ipv4_address[1], expected 241, is %d",
                  check_msg->ipv4_address[1]);
    ck_assert_msg(check_msg->ipv4_address[2] == 84,
                  "incorrect value for ipv4_address[2], expected 84, is %d",
                  check_msg->ipv4_address[2]);
    ck_assert_msg(check_msg->ipv4_address[3] == 180,
                  "incorrect value for ipv4_address[3], expected 180, is %d",
                  check_msg->ipv4_address[3]);
    ck_assert_msg(check_msg->ipv4_mask_size == 152,
                  "incorrect value for ipv4_mask_size, expected 152, is %d",
                  check_msg->ipv4_mask_size);
    ck_assert_msg(check_msg->ipv6_address[0] == 194,
                  "incorrect value for ipv6_address[0], expected 194, is %d",
                  check_msg->ipv6_address[0]);
    ck_assert_msg(check_msg->ipv6_address[1] == 137,
                  "incorrect value for ipv6_address[1], expected 137, is %d",
                  check_msg->ipv6_address[1]);
    ck_assert_msg(check_msg->ipv6_address[2] == 32,
                  "incorrect value for ipv6_address[2], expected 32, is %d",
                  check_msg->ipv6_address[2]);
    ck_assert_msg(check_msg->ipv6_address[3] == 44,
                  "incorrect value for ipv6_address[3], expected 44, is %d",
                  check_msg->ipv6_address[3]);
    ck_assert_msg(check_msg->ipv6_address[4] == 114,
                  "incorrect value for ipv6_address[4], expected 114, is %d",
                  check_msg->ipv6_address[4]);
    ck_assert_msg(check_msg->ipv6_address[5] == 147,
                  "incorrect value for ipv6_address[5], expected 147, is %d",
                  check_msg->ipv6_address[5]);
    ck_assert_msg(check_msg->ipv6_address[6] == 68,
                  "incorrect value for ipv6_address[6], expected 68, is %d",
                  check_msg->ipv6_address[6]);
    ck_assert_msg(check_msg->ipv6_address[7] == 222,
                  "incorrect value for ipv6_address[7], expected 222, is %d",
                  check_msg->ipv6_address[7]);
    ck_assert_msg(check_msg->ipv6_address[8] == 92,
                  "incorrect value for ipv6_address[8], expected 92, is %d",
                  check_msg->ipv6_address[8]);
    ck_assert_msg(check_msg->ipv6_address[9] == 192,
                  "incorrect value for ipv6_address[9], expected 192, is %d",
                  check_msg->ipv6_address[9]);
    ck_assert_msg(check_msg->ipv6_address[10] == 78,
                  "incorrect value for ipv6_address[10], expected 78, is %d",
                  check_msg->ipv6_address[10]);
    ck_assert_msg(check_msg->ipv6_address[11] == 235,
                  "incorrect value for ipv6_address[11], expected 235, is %d",
                  check_msg->ipv6_address[11]);
    ck_assert_msg(check_msg->ipv6_address[12] == 63,
                  "incorrect value for ipv6_address[12], expected 63, is %d",
                  check_msg->ipv6_address[12]);
    ck_assert_msg(check_msg->ipv6_address[13] == 208,
                  "incorrect value for ipv6_address[13], expected 208, is %d",
                  check_msg->ipv6_address[13]);
    ck_assert_msg(check_msg->ipv6_address[14] == 114,
                  "incorrect value for ipv6_address[14], expected 114, is %d",
                  check_msg->ipv6_address[14]);
    ck_assert_msg(check_msg->ipv6_address[15] == 53,
                  "incorrect value for ipv6_address[15], expected 53, is %d",
                  check_msg->ipv6_address[15]);
    ck_assert_msg(check_msg->ipv6_mask_size == 183,
                  "incorrect value for ipv6_mask_size, expected 183, is %d",
                  check_msg->ipv6_mask_size);
    ck_assert_msg(check_msg->rx_bytes == 451408920,
                  "incorrect value for rx_bytes, expected 451408920, is %d",
                  check_msg->rx_bytes);
    ck_assert_msg(check_msg->tx_bytes == 59251049,
                  "incorrect value for tx_bytes, expected 59251049, is %d",
                  check_msg->tx_bytes);
  }
}
END_TEST

Suite *legacy_auto_check_sbp_piksi_MsgNetworkStateResp_suite(void) {
  Suite *s = suite_create(
      "SBP generated test suite: "
      "legacy_auto_check_sbp_piksi_MsgNetworkStateResp");
  TCase *tc_acq = tcase_create(
      "Automated_Suite_legacy_auto_check_sbp_piksi_MsgNetworkStateResp");
  tcase_add_test(tc_acq, test_legacy_auto_check_sbp_piksi_MsgNetworkStateResp);
  suite_add_tcase(s, tc_acq);
  return s;
}