#!/usr/bin/env python
# Copyright (C) 2015-2023 Swift Navigation Inc.
# Contact: https://support.swiftnav.com
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.

"""
Generator for Kaitai Struct perl tests.

This module consumes the YAML test spec and generates test caes for the
perl bindings generated by kaitai-struct-compiler
"""

from sbpg.targets.common import string_type, dict_type, array_type, snake_case, snake_case_keys, encode_json, decode_json, str_escape
from sbpg.targets.templating import JENV

TEMPLATE_NAME = "kaitai/test_perl.t.j2"

JENV.filters['snake_case'] = snake_case
JENV.filters['snake_case_keys'] = snake_case_keys
JENV.filters['str_escape'] = str_escape
JENV.filters['encode_json'] = encode_json
JENV.filters['decode_json'] = decode_json

JENV.tests['string_type'] = string_type
JENV.tests['dict_type'] = dict_type
JENV.tests['array_type'] = array_type


def render_source(output_dir, package_spec, jenv=JENV):
  """
  Render and output a .t file containing test cases
  """
  path, name = package_spec.filepath
  destination_filename = "{}/{}.t".format(output_dir, name)
  test_template = JENV.get_template(TEMPLATE_NAME)
  with open(destination_filename, 'w') as f:
    f.write(test_template.render(s=package_spec))
