#!/usr/bin/env python
# Copyright (C) 2015-2023 Swift Navigation Inc.
# Contact: https://support.swiftnav.com
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.

"""
Generator for Kaitai Struct python tests.

This module consumes the YAML test spec and generates test caes for the
python bindings generated by kaitai-struct-compiler
"""

from sbpg.targets.common import string_type, snake_case, snake_case_keys, str_escape, decode_json
from sbpg.targets.templating import JENV

TEMPLATE_NAME = "kaitai/test_python.py.j2"

JENV.filters['snake_case'] = snake_case
JENV.filters['snake_case_keys'] = snake_case_keys
JENV.filters['str_escape'] = str_escape
JENV.filters['decode_json'] = decode_json

JENV.tests['string_type'] = string_type


def render_source(output_dir, package_spec, jenv=JENV):
  """
  Render and output a .py file containing test cases
  """
  path, name = package_spec.filepath
  destination_filename = "{}/test_{}.py".format(output_dir, name)
  test_template = JENV.get_template(TEMPLATE_NAME)
  with open(destination_filename, 'w') as f:
    f.write(test_template.render(s=package_spec))
