/* Copyright (C) 2015-2022 Swift Navigation Inc.
 * Contact: https://support.swiftnav.com
 *
 * This source is subject to the license found in the file 'LICENSE' which must
 * be be distributed together with this source. All other rights reserved.
 *
 * THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
 * EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
 */
package com.swiftnav.sbp.orientation;

// This file was auto-generated from yaml/swiftnav/sbp/orientation.yaml by generate.py.
// Do not modify by hand!


import com.swiftnav.sbp.SBPBinaryException;
import com.swiftnav.sbp.SBPMessage;
import org.json.JSONObject;

/**
 * SBP class for message MSG_ANGULAR_RATE (0x0222).
 *
 * <p>You can have MSG_ANGULAR_RATE inherent its fields directly from an inherited SBP object, or
 * construct it inline using a dict of its fields.
 *
 * <p>This message reports the orientation rates in the vehicle body frame. The values represent the
 * measurements a strapped down gyroscope would make and are not equivalent to the time derivative
 * of the Euler angles. The orientation and origin of the user frame is specified via device
 * settings. By convention, the vehicle x-axis is expected to be aligned with the forward direction,
 * while the vehicle y-axis is expected to be aligned with the right direction, and the vehicle
 * z-axis should be aligned with the down direction. This message will only be available in future
 * INS versions of Swift Products and is not produced by Piksi Multi or Duro.
 */
public class MsgAngularRate extends SBPMessage {
    public static final int TYPE = 0x0222;

    /** GPS Time of Week */
    public long tow;

    /** angular rate about x axis */
    public int x;

    /** angular rate about y axis */
    public int y;

    /** angular rate about z axis */
    public int z;

    /** Status flags */
    public int flags;

    public MsgAngularRate(int sender) {
        super(sender, TYPE);
    }

    public MsgAngularRate() {
        super(TYPE);
    }

    public MsgAngularRate(SBPMessage msg) throws SBPBinaryException {
        super(msg);
        if (msg.type != TYPE)
            throw new SBPBinaryException(
                    "Type mismatch for MsgAngularRate, expected 546, actual " + msg.type);
    }

    @Override
    protected void parse(Parser parser) throws SBPBinaryException {
        /* Parse fields from binary */
        tow = parser.getU32();
        x = parser.getS32();
        y = parser.getS32();
        z = parser.getS32();
        flags = parser.getU8();
    }

    @Override
    protected void build(Builder builder) {
        builder.putU32(tow);
        builder.putS32(x);
        builder.putS32(y);
        builder.putS32(z);
        builder.putU8(flags);
    }

    @Override
    public JSONObject toJSON() {
        JSONObject obj = super.toJSON();
        obj.put("tow", tow);
        obj.put("x", x);
        obj.put("y", y);
        obj.put("z", z);
        obj.put("flags", flags);
        return obj;
    }

    @Override
    public String getFriendlyName() {
        return "ANGULAR RATE";
    }
}
