/**
 * Copyright (C) 2015-2021 Swift Navigation Inc.
 * Contact: https://support.swiftnav.com
 * This source is subject to the license found in the file 'LICENSE' which must
 * be distributed together with this source. All other rights reserved.
 *
 * THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
 * EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
 */

/**********************
 * Automatically generated from spec/yaml/swiftnav/sbp/file_io.yaml with generate.py.
 * Don't edit this by hand!
 **********************
 * Package description:
 *
 * Messages for using device's onboard flash filesystem functionality. This allows
 * data to be stored persistently in the device's program flash with wear-levelling
 * using a simple filesystem interface. The file system interface (CFS) defines an
 * abstract API for reading directories and for reading and writing files.  Note
 * that some of these messages share the same message type ID for both the host
 * request and the device response.
***********************/

let SBP = require('./sbp');
let Parser = require('./parser');
let Int64 = require('node-int64');
let UInt64 = require('cuint').UINT64;

/**
 * SBP class for message MSG_FILEIO_READ_REQ (0x00A8).
 *
 * The file read message reads a certain length (up to 255 bytes) from a given
 * offset into a file, and returns the data in a MSG_FILEIO_READ_RESP message where
 * the message length field indicates how many bytes were successfully read. The
 * sequence number in the request will be returned in the response. If the message
 * is invalid, a followup MSG_PRINT message will print "Invalid fileio read
 * message". A device will only respond to this message when it is received from
 * sender ID 0x42.
 *
 * Fields in the SBP payload (`sbp.payload`):
 * @field sequence number (unsigned 32-bit int, 4 bytes) Read sequence number
 * @field offset number (unsigned 32-bit int, 4 bytes) File offset
 * @field chunk_size number (unsigned 8-bit int, 1 byte) Chunk size to read
 * @field filename string Name of the file to read from
 *
 * @param sbp An SBP object with a payload to be decoded.
 */
let MsgFileioReadReq = function (sbp, fields) {
  SBP.call(this, sbp);
  this.messageType = "MSG_FILEIO_READ_REQ";
  this.fields = (fields || this.parser.parse(sbp.payload));

  return this;
};
MsgFileioReadReq.prototype = Object.create(SBP.prototype);
MsgFileioReadReq.prototype.messageType = "MSG_FILEIO_READ_REQ";
MsgFileioReadReq.prototype.msg_type = 0x00A8;
MsgFileioReadReq.prototype.constructor = MsgFileioReadReq;
MsgFileioReadReq.prototype.parser = new Parser()
  .endianess('little')
  .uint32('sequence')
  .uint32('offset')
  .uint8('chunk_size')
  .string('filename', { greedy: true });
MsgFileioReadReq.prototype.fieldSpec = [];
MsgFileioReadReq.prototype.fieldSpec.push(['sequence', 'writeUInt32LE', 4]);
MsgFileioReadReq.prototype.fieldSpec.push(['offset', 'writeUInt32LE', 4]);
MsgFileioReadReq.prototype.fieldSpec.push(['chunk_size', 'writeUInt8', 1]);
MsgFileioReadReq.prototype.fieldSpec.push(['filename', 'string', null]);

/**
 * SBP class for message MSG_FILEIO_READ_RESP (0x00A3).
 *
 * The file read message reads a certain length (up to 255 bytes) from a given
 * offset into a file, and returns the data in a message where the message length
 * field indicates how many bytes were successfully read. The sequence number in
 * the response is preserved from the request.
 *
 * Fields in the SBP payload (`sbp.payload`):
 * @field sequence number (unsigned 32-bit int, 4 bytes) Read sequence number
 * @field contents array Contents of read file
 *
 * @param sbp An SBP object with a payload to be decoded.
 */
let MsgFileioReadResp = function (sbp, fields) {
  SBP.call(this, sbp);
  this.messageType = "MSG_FILEIO_READ_RESP";
  this.fields = (fields || this.parser.parse(sbp.payload));

  return this;
};
MsgFileioReadResp.prototype = Object.create(SBP.prototype);
MsgFileioReadResp.prototype.messageType = "MSG_FILEIO_READ_RESP";
MsgFileioReadResp.prototype.msg_type = 0x00A3;
MsgFileioReadResp.prototype.constructor = MsgFileioReadResp;
MsgFileioReadResp.prototype.parser = new Parser()
  .endianess('little')
  .uint32('sequence')
  .array('contents', { type: 'uint8', readUntil: 'eof' });
MsgFileioReadResp.prototype.fieldSpec = [];
MsgFileioReadResp.prototype.fieldSpec.push(['sequence', 'writeUInt32LE', 4]);
MsgFileioReadResp.prototype.fieldSpec.push(['contents', 'array', 'writeUInt8', function () { return 1; }, null]);

/**
 * SBP class for message MSG_FILEIO_READ_DIR_REQ (0x00A9).
 *
 * The read directory message lists the files in a directory on the device's
 * onboard flash file system.  The offset parameter can be used to skip the first n
 * elements of the file list. Returns a MSG_FILEIO_READ_DIR_RESP message containing
 * the directory listings as a NULL delimited list. The listing is chunked over
 * multiple SBP packets. The sequence number in the request will be returned in the
 * response.  If message is invalid, a followup MSG_PRINT message will print
 * "Invalid fileio read message". A device will only respond to this message when
 * it is received from sender ID 0x42.
 *
 * Fields in the SBP payload (`sbp.payload`):
 * @field sequence number (unsigned 32-bit int, 4 bytes) Read sequence number
 * @field offset number (unsigned 32-bit int, 4 bytes) The offset to skip the first n elements of the file list
 * @field dirname string Name of the directory to list
 *
 * @param sbp An SBP object with a payload to be decoded.
 */
let MsgFileioReadDirReq = function (sbp, fields) {
  SBP.call(this, sbp);
  this.messageType = "MSG_FILEIO_READ_DIR_REQ";
  this.fields = (fields || this.parser.parse(sbp.payload));

  return this;
};
MsgFileioReadDirReq.prototype = Object.create(SBP.prototype);
MsgFileioReadDirReq.prototype.messageType = "MSG_FILEIO_READ_DIR_REQ";
MsgFileioReadDirReq.prototype.msg_type = 0x00A9;
MsgFileioReadDirReq.prototype.constructor = MsgFileioReadDirReq;
MsgFileioReadDirReq.prototype.parser = new Parser()
  .endianess('little')
  .uint32('sequence')
  .uint32('offset')
  .string('dirname', { greedy: true });
MsgFileioReadDirReq.prototype.fieldSpec = [];
MsgFileioReadDirReq.prototype.fieldSpec.push(['sequence', 'writeUInt32LE', 4]);
MsgFileioReadDirReq.prototype.fieldSpec.push(['offset', 'writeUInt32LE', 4]);
MsgFileioReadDirReq.prototype.fieldSpec.push(['dirname', 'string', null]);

/**
 * SBP class for message MSG_FILEIO_READ_DIR_RESP (0x00AA).
 *
 * The read directory message lists the files in a directory on the device's
 * onboard flash file system. Message contains the directory listings as a NULL
 * delimited list. The listing is chunked over multiple SBP packets and the end of
 * the list is identified by an packet with no entries. The sequence number in the
 * response is preserved from the request.
 *
 * Fields in the SBP payload (`sbp.payload`):
 * @field sequence number (unsigned 32-bit int, 4 bytes) Read sequence number
 * @field contents string Contents of read directory
 *
 * @param sbp An SBP object with a payload to be decoded.
 */
let MsgFileioReadDirResp = function (sbp, fields) {
  SBP.call(this, sbp);
  this.messageType = "MSG_FILEIO_READ_DIR_RESP";
  this.fields = (fields || this.parser.parse(sbp.payload));

  return this;
};
MsgFileioReadDirResp.prototype = Object.create(SBP.prototype);
MsgFileioReadDirResp.prototype.messageType = "MSG_FILEIO_READ_DIR_RESP";
MsgFileioReadDirResp.prototype.msg_type = 0x00AA;
MsgFileioReadDirResp.prototype.constructor = MsgFileioReadDirResp;
MsgFileioReadDirResp.prototype.parser = new Parser()
  .endianess('little')
  .uint32('sequence')
  .string('contents', { greedy: true });
MsgFileioReadDirResp.prototype.fieldSpec = [];
MsgFileioReadDirResp.prototype.fieldSpec.push(['sequence', 'writeUInt32LE', 4]);
MsgFileioReadDirResp.prototype.fieldSpec.push(['contents', 'string', null]);

/**
 * SBP class for message MSG_FILEIO_REMOVE (0x00AC).
 *
 * The file remove message deletes a file from the file system. If the message is
 * invalid, a followup MSG_PRINT message will print "Invalid fileio remove
 * message". A device will only process this message when it is received from
 * sender ID 0x42.
 *
 * Fields in the SBP payload (`sbp.payload`):
 * @field filename string Name of the file to delete
 *
 * @param sbp An SBP object with a payload to be decoded.
 */
let MsgFileioRemove = function (sbp, fields) {
  SBP.call(this, sbp);
  this.messageType = "MSG_FILEIO_REMOVE";
  this.fields = (fields || this.parser.parse(sbp.payload));

  return this;
};
MsgFileioRemove.prototype = Object.create(SBP.prototype);
MsgFileioRemove.prototype.messageType = "MSG_FILEIO_REMOVE";
MsgFileioRemove.prototype.msg_type = 0x00AC;
MsgFileioRemove.prototype.constructor = MsgFileioRemove;
MsgFileioRemove.prototype.parser = new Parser()
  .endianess('little')
  .string('filename', { greedy: true });
MsgFileioRemove.prototype.fieldSpec = [];
MsgFileioRemove.prototype.fieldSpec.push(['filename', 'string', null]);

/**
 * SBP class for message MSG_FILEIO_WRITE_REQ (0x00AD).
 *
 * The file write message writes a certain length (up to 255 bytes) of data to a
 * file at a given offset. Returns a copy of the original MSG_FILEIO_WRITE_RESP
 * message to check integrity of the write. The sequence number in the request will
 * be returned in the response. If message is invalid, a followup MSG_PRINT message
 * will print "Invalid fileio write message". A device will only process this
 * message when it is received from sender ID 0x42.
 *
 * Fields in the SBP payload (`sbp.payload`):
 * @field sequence number (unsigned 32-bit int, 4 bytes) Write sequence number
 * @field offset number (unsigned 32-bit int, 4 bytes) Offset into the file at which to start writing in bytes
 * @field filename string Name of the file to write to
 * @field data array Variable-length array of data to write
 *
 * @param sbp An SBP object with a payload to be decoded.
 */
let MsgFileioWriteReq = function (sbp, fields) {
  SBP.call(this, sbp);
  this.messageType = "MSG_FILEIO_WRITE_REQ";
  this.fields = (fields || this.parser.parse(sbp.payload));

  return this;
};
MsgFileioWriteReq.prototype = Object.create(SBP.prototype);
MsgFileioWriteReq.prototype.messageType = "MSG_FILEIO_WRITE_REQ";
MsgFileioWriteReq.prototype.msg_type = 0x00AD;
MsgFileioWriteReq.prototype.constructor = MsgFileioWriteReq;
MsgFileioWriteReq.prototype.parser = new Parser()
  .endianess('little')
  .uint32('sequence')
  .uint32('offset')
  .string('filename', { greedy: true })
  .array('data', { type: 'uint8', readUntil: 'eof' });
MsgFileioWriteReq.prototype.fieldSpec = [];
MsgFileioWriteReq.prototype.fieldSpec.push(['sequence', 'writeUInt32LE', 4]);
MsgFileioWriteReq.prototype.fieldSpec.push(['offset', 'writeUInt32LE', 4]);
MsgFileioWriteReq.prototype.fieldSpec.push(['filename', 'string', null]);
MsgFileioWriteReq.prototype.fieldSpec.push(['data', 'array', 'writeUInt8', function () { return 1; }, null]);

/**
 * SBP class for message MSG_FILEIO_WRITE_RESP (0x00AB).
 *
 * The file write message writes a certain length (up to 255 bytes) of data to a
 * file at a given offset. The message is a copy of the original
 * MSG_FILEIO_WRITE_REQ message to check integrity of the write. The sequence
 * number in the response is preserved from the request.
 *
 * Fields in the SBP payload (`sbp.payload`):
 * @field sequence number (unsigned 32-bit int, 4 bytes) Write sequence number
 *
 * @param sbp An SBP object with a payload to be decoded.
 */
let MsgFileioWriteResp = function (sbp, fields) {
  SBP.call(this, sbp);
  this.messageType = "MSG_FILEIO_WRITE_RESP";
  this.fields = (fields || this.parser.parse(sbp.payload));

  return this;
};
MsgFileioWriteResp.prototype = Object.create(SBP.prototype);
MsgFileioWriteResp.prototype.messageType = "MSG_FILEIO_WRITE_RESP";
MsgFileioWriteResp.prototype.msg_type = 0x00AB;
MsgFileioWriteResp.prototype.constructor = MsgFileioWriteResp;
MsgFileioWriteResp.prototype.parser = new Parser()
  .endianess('little')
  .uint32('sequence');
MsgFileioWriteResp.prototype.fieldSpec = [];
MsgFileioWriteResp.prototype.fieldSpec.push(['sequence', 'writeUInt32LE', 4]);

/**
 * SBP class for message MSG_FILEIO_CONFIG_REQ (0x1001).
 *
 * Requests advice on the optimal configuration for a FileIO transfer.  Newer
 * version of FileIO can support greater throughput by supporting a large window of
 * FileIO data that can be in-flight during read or write operations.
 *
 * Fields in the SBP payload (`sbp.payload`):
 * @field sequence number (unsigned 32-bit int, 4 bytes) Advice sequence number
 *
 * @param sbp An SBP object with a payload to be decoded.
 */
let MsgFileioConfigReq = function (sbp, fields) {
  SBP.call(this, sbp);
  this.messageType = "MSG_FILEIO_CONFIG_REQ";
  this.fields = (fields || this.parser.parse(sbp.payload));

  return this;
};
MsgFileioConfigReq.prototype = Object.create(SBP.prototype);
MsgFileioConfigReq.prototype.messageType = "MSG_FILEIO_CONFIG_REQ";
MsgFileioConfigReq.prototype.msg_type = 0x1001;
MsgFileioConfigReq.prototype.constructor = MsgFileioConfigReq;
MsgFileioConfigReq.prototype.parser = new Parser()
  .endianess('little')
  .uint32('sequence');
MsgFileioConfigReq.prototype.fieldSpec = [];
MsgFileioConfigReq.prototype.fieldSpec.push(['sequence', 'writeUInt32LE', 4]);

/**
 * SBP class for message MSG_FILEIO_CONFIG_RESP (0x1002).
 *
 * The advice on the optimal configuration for a FileIO transfer.  Newer version of
 * FileIO can support greater throughput by supporting a large window of FileIO
 * data that can be in-flight during read or write operations.
 *
 * Fields in the SBP payload (`sbp.payload`):
 * @field sequence number (unsigned 32-bit int, 4 bytes) Advice sequence number
 * @field window_size number (unsigned 32-bit int, 4 bytes) The number of SBP packets in the data in-flight window
 * @field batch_size number (unsigned 32-bit int, 4 bytes) The number of SBP packets sent in one PDU
 * @field fileio_version number (unsigned 32-bit int, 4 bytes) The version of FileIO that is supported
 *
 * @param sbp An SBP object with a payload to be decoded.
 */
let MsgFileioConfigResp = function (sbp, fields) {
  SBP.call(this, sbp);
  this.messageType = "MSG_FILEIO_CONFIG_RESP";
  this.fields = (fields || this.parser.parse(sbp.payload));

  return this;
};
MsgFileioConfigResp.prototype = Object.create(SBP.prototype);
MsgFileioConfigResp.prototype.messageType = "MSG_FILEIO_CONFIG_RESP";
MsgFileioConfigResp.prototype.msg_type = 0x1002;
MsgFileioConfigResp.prototype.constructor = MsgFileioConfigResp;
MsgFileioConfigResp.prototype.parser = new Parser()
  .endianess('little')
  .uint32('sequence')
  .uint32('window_size')
  .uint32('batch_size')
  .uint32('fileio_version');
MsgFileioConfigResp.prototype.fieldSpec = [];
MsgFileioConfigResp.prototype.fieldSpec.push(['sequence', 'writeUInt32LE', 4]);
MsgFileioConfigResp.prototype.fieldSpec.push(['window_size', 'writeUInt32LE', 4]);
MsgFileioConfigResp.prototype.fieldSpec.push(['batch_size', 'writeUInt32LE', 4]);
MsgFileioConfigResp.prototype.fieldSpec.push(['fileio_version', 'writeUInt32LE', 4]);

module.exports = {
  0x00A8: MsgFileioReadReq,
  MsgFileioReadReq: MsgFileioReadReq,
  0x00A3: MsgFileioReadResp,
  MsgFileioReadResp: MsgFileioReadResp,
  0x00A9: MsgFileioReadDirReq,
  MsgFileioReadDirReq: MsgFileioReadDirReq,
  0x00AA: MsgFileioReadDirResp,
  MsgFileioReadDirResp: MsgFileioReadDirResp,
  0x00AC: MsgFileioRemove,
  MsgFileioRemove: MsgFileioRemove,
  0x00AD: MsgFileioWriteReq,
  MsgFileioWriteReq: MsgFileioWriteReq,
  0x00AB: MsgFileioWriteResp,
  MsgFileioWriteResp: MsgFileioWriteResp,
  0x1001: MsgFileioConfigReq,
  MsgFileioConfigReq: MsgFileioConfigReq,
  0x1002: MsgFileioConfigResp,
  MsgFileioConfigResp: MsgFileioConfigResp,
}